// Exercice sur la resolution d'un sudoku
//  

// Definition de la case de la grille de Sudoku
// ===================================================
class Case
{
    int val;       // Valeur entre 0 et 9 (si 0 alors case vide)
    boolean fixe;  // La valeur est fixe (grille de dpart)
}

// Definition de la grille de Sudoku
// ===================================================
class Grille
{
    Case[][] grille;  // La grille (tableau a 2 dimension de Case)

    // Constructeur
    public Grille()
    {
        // Creation de la grille
        grille = new Case[9][9];
    }

    // Affiche la grille a l'ecran
    public void afficher()
    {
        for(int l=0;l<9;l++){
            for(int c=0;c<9;c++)
                System.out.print(grille[l][c].val+" ");
                System.out.println();
        }
        System.out.println("-------------------------------");
    }

    // Algorithme de resolution d'une grille de Sudoku
    // L'algorithme s'arrete a la 1ere solution trouvee
    //
    public void resoudre()
    {
        boolean fini = false;
        int p=0;  // Parcours de la 1ere case (en haut a gauche) jusqu'a la
        // derniere case (en bas a droite) avec un parcours du gauche a droite
        // et de bas en haut
        // 
        while(!fini){
            // Si la case courante est fixe alors passer a la suivante
            if (fixe(p)) 
                {
                    if (p==80) fini=true; // On est sur la derniere case
                    // on arrete
                    else p++; // case suivante
                }
            else {
                // Si la case contient la valeur 9, il faut revenir en arriere
                if (val(p)==9){
                    val(p,0);
                    if (p==0) fini=true;
                    else
                        {
                            p--;
                            while ( (p>=0) && (fixe(p))) p--;
                            if (p<0) fini=true;
                        }
                }
                // sinon on incremente la valeur de la case, et si elle est
                //  valide on passe a la suivante sinon rien
                else{
                    val(p,val(p)+1);
                    if (valide(p)){
                        if (p==80) fini=true;
                        else p++;
                    }
                }
            }
        }
    }

        
    // Dans les methodes qui suivent, la position p est un entier de 0  80
    // qui est le rang de la case dans la grille suivant un parcours de 
    // gauche a droite et de haut en bas

    // Retourne la valeur de la case se trouvant a la position p
    private int val(int p){
        return grille[p/9][p%9].val;
    }
    // Change la valeur de la case se trouvant a la position p
    private void val(int p,int v){
        grille[p/9][p%9].val=v;
    }
    // Retourne si fixe ou non de la case se trouvant a la position p
    private boolean fixe(int p){
        return grille[p/9][p%9].fixe;
    }

    // Teste si la valeur de la case se trouvant a la position p est valide
    // C est a dire s'il n'existe pas une meme valeur dans la ligne, ou la
    // colonne, ou le bloc 3x3
    //
    private boolean valide(int p)
    {
        // Conversion de la position en coordonnees dans la grille
        int l = p/9; 
        int c = p%9;

        // La valeur a tester
        int v=grille[l][c].val;
        
        // Teste sur la ligne
        for(int i=0;i<9;i++)
            if ( (i!=c)&&(grille[l][i].val==v) ) return false;

        // Teste sur la colonne
        for(int i=0;i<9;i++)
            if ( (i!=l)&&(grille[i][c].val==v) ) return false;

        // Teste dans le bloc 3x3
        for(int i=(l/3)*3;i<(l/3)*3+3;i++)
            for(int j=(c/3)*3;j<(c/3)*3+3;j++)
                if ( ( (i!=l) || (j!=c) ) && (grille[i][j].val==v) ) return false;
        
        // Pour tracer le deroulement de la resolution
        // afficher();
        // Pour debuguer pas a pas
        //Terminal.lireString();

        return true;
    }
    
}

// Programme principal
public class Exercice08
{
    /** mthode principale de l'exercice */
    public static void main(String... args)
    {
        // Creation de la grille de sudoku
        Grille sudo = new Grille();

        if (args.length ==1)
            {
                // Initialisation de la grille avec celle en entree
                // La valeur en entree est 1 chaine de caractere dont chaque
                // caractere est la valeur d'une case
                //
                int l=0;
                int c=0;
                for(int k=0;k<81;k++)
                    {
                        sudo.grille[l][c] = new Case();
                        sudo.grille[l][c].val=Integer.parseInt(args[0].charAt(k)+"");
                        if (sudo.grille[l][c].val!=0) sudo.grille[l][c].fixe=true;
                        
                        if ((c!=0)&&(c%8==0)){l++;c=0;}
                        else c++;
                    }
            }
        else
            {
                int l=0;
                int c=0;
                for(int k=0;k<81;k++)
                    {
                        sudo.grille[l][c] = new Case();
                        sudo.grille[l][c].val=0;
                        sudo.grille[l][c].fixe=false;
                        
                        if ((c!=0)&&(c%8==0)){l++;c=0;}
                        else c++;
                    }
            }

        // Si pas de parametre alors la grille initiale est vide.

        // Affichage de la grille d'entree
        sudo.afficher();

        // Resolution de la grille
        sudo.resoudre();

        // Affichage de la solution
        sudo.afficher();
    }
}
